@@ -66,7 +66,6 @@ from pyfakefs import fake_pathlib
 from pyfakefs import mox3_stubout
 from pyfakefs.extra_packages import pathlib, pathlib2, use_scandir
 
-
 if use_scandir:
     from pyfakefs import fake_scandir
 
@@ -80,15 +79,17 @@ def patchfs(_func=None, *,
             modules_to_patch=None,
             allow_root_user=True,
             use_known_patches=True,
-            patch_open_code=PatchMode.OFF):
+            patch_open_code=PatchMode.OFF,
+            patch_default_args=False,
+            use_cache=True):
     """Convenience decorator to use patcher with additional parameters in a
     test function.
 
     Usage::
 
         @patchfs
-        def test_my_function(fs):
-            fs.create_file('foo')
+        def test_my_function(fake_fs):
+            fake_fs.create_file('foo')
 
         @patchfs(allow_root_user=False)
         def test_with_patcher_args(fs):
@@ -104,8 +105,11 @@ def patchfs(_func=None, *,
                     modules_to_patch=modules_to_patch,
                     allow_root_user=allow_root_user,
                     use_known_patches=use_known_patches,
-                    patch_open_code=patch_open_code) as p:
-                kwargs['fs'] = p.fs
+                    patch_open_code=patch_open_code,
+                    patch_default_args=patch_default_args,
+                    use_cache=use_cache) as p:
+                args = list(args)
+                args.append(p.fs)
                 return f(*args, **kwargs)
 
         return wrapped
@@ -113,9 +117,11 @@ def patchfs(_func=None, *,
     if _func:
         if not callable(_func):
             raise TypeError(
-                "Decorator argument not a function.\n"
+                "Decorator argument is not a function.\n"
                 "Did you mean `@patchfs(additional_skip_names=...)`?"
             )
+        if hasattr(_func, 'patchings'):
+            _func.nr_patches = len(_func.patchings)
         return wrap_patchfs(_func)
 
     return wrap_patchfs
@@ -128,7 +134,8 @@ def load_doctests(
         modules_to_patch=None,
         allow_root_user=True,
         use_known_patches=True,
-        patch_open_code=PatchMode.OFF):  # pylint: disable=unused-argument
+        patch_open_code=PatchMode.OFF,
+        patch_default_args=False):  # pylint: disable=unused-argument
     """Load the doctest tests for the specified module into unittest.
         Args:
             loader, tests, ignore : arguments passed in from `load_tests()`
@@ -142,7 +149,8 @@ def load_doctests(
                        modules_to_patch=modules_to_patch,
                        allow_root_user=allow_root_user,
                        use_known_patches=use_known_patches,
-                       patch_open_code=patch_open_code)
+                       patch_open_code=patch_open_code,
+                       patch_default_args=patch_default_args)
     globs = _patcher.replace_globs(vars(module))
     tests.addTests(doctest.DocTestSuite(module,
                                         globs=globs,
@@ -205,7 +213,9 @@ class TestCaseMixin:
                       modules_to_patch=None,
                       allow_root_user=True,
                       use_known_patches=True,
-                      patch_open_code=PatchMode.OFF):
+                      patch_open_code=PatchMode.OFF,
+                      patch_default_args=False,
+                      use_cache=True):
         """Bind the file-related modules to the :py:class:`pyfakefs` fake file
         system instead of the real file system.  Also bind the fake `open()`
         function.
@@ -229,7 +239,9 @@ class TestCaseMixin:
             modules_to_patch=modules_to_patch,
             allow_root_user=allow_root_user,
             use_known_patches=use_known_patches,
-            patch_open_code=patch_open_code
+            patch_open_code=patch_open_code,
+            patch_default_args=patch_default_args,
+            use_cache=use_cache
         )
 
         self._stubber.setUp()
@@ -353,13 +365,13 @@ class Patcher:
         None, fake_filesystem, fake_filesystem_shutil,
         sys, linecache, tokenize
     }
-    try:
-        import py
-        import pytest
-        SKIPMODULES.add(py)
-        SKIPMODULES.add(pytest)
-    except ImportError:
-        pass
+    # caches all modules that do not have file system modules or function
+    # to speed up _find_modules
+    CACHED_MODULES = set()
+    FS_MODULES = {}
+    FS_FUNCTIONS = {}
+    FS_DEFARGS = []
+    SKIPPED_FS_MODULES = {}
 
     assert None in SKIPMODULES, ("sys.modules contains 'None' values;"
                                  " must skip them.")
@@ -368,10 +380,17 @@ class Patcher:
 
     SKIPNAMES = {'os', 'path', 'io', 'genericpath', OS_MODULE, PATH_MODULE}
 
+    # hold values from last call - if changed, the cache has to be invalidated
+    PATCHED_MODULE_NAMES = {}
+    ADDITIONAL_SKIP_NAMES = set()
+    PATCH_DEFAULT_ARGS = False
+
     def __init__(self, additional_skip_names=None,
                  modules_to_reload=None, modules_to_patch=None,
                  allow_root_user=True, use_known_patches=True,
-                 patch_open_code=PatchMode.OFF):
+                 patch_open_code=PatchMode.OFF,
+                 patch_default_args=False,
+                 use_cache=True):
         """
         Args:
             additional_skip_names: names of modules inside of which no module
@@ -395,6 +414,13 @@ class Patcher:
             patch_open_code: If True, `io.open_code` is patched. The default
                 is not to patch it, as it mostly is used to load compiled
                 modules that are not in the fake file system.
+            patch_default_args: If True, default arguments are checked for
+                file system functions, which are patched. This check is
+                expansive, so it is off by default.
+            use_cache: If True (default), patched and non-patched modules are
+                cached between tests for performance reasons. As this is a new
+                feature, this argument allows to turn it off in case it
+                causes any problems.
         """
 
         if not allow_root_user:
@@ -405,7 +431,6 @@ class Patcher:
         self._skip_names = self.SKIPNAMES.copy()
         # save the original open function for use in pytest plugin
         self.original_open = open
-        self.fake_open = None
         self.patch_open_code = patch_open_code
 
         if additional_skip_names is not None:
@@ -418,7 +443,12 @@ class Patcher:
         self._class_modules = {}
         self._init_fake_module_classes()
 
-        self.modules_to_reload = modules_to_reload or []
+        # reload tempfile under posix to patch default argument
+        self.modules_to_reload = [] if sys.platform == 'win32' else [tempfile]
+        if modules_to_reload is not None:
+            self.modules_to_reload.extend(modules_to_reload)
+        self.patch_default_args = patch_default_args
+        self.use_cache = use_cache
 
         if use_known_patches:
             modules_to_patch = modules_to_patch or {}
@@ -429,26 +459,43 @@ class Patcher:
         if modules_to_patch is not None:
             for name, fake_module in modules_to_patch.items():
                 self._fake_module_classes[name] = fake_module
+        patched_module_names = set(modules_to_patch)
+        clear_cache = not use_cache
+        if use_cache:
+            if patched_module_names != self.PATCHED_MODULE_NAMES:
+                self.__class__.PATCHED_MODULE_NAMES = patched_module_names
+                clear_cache = True
+            if self._skip_names != self.ADDITIONAL_SKIP_NAMES:
+                self.__class__.ADDITIONAL_SKIP_NAMES = self._skip_names
+                clear_cache = True
+            if patch_default_args != self.PATCH_DEFAULT_ARGS:
+                self.__class__.PATCH_DEFAULT_ARGS = patch_default_args
+                clear_cache = True
 
+        if clear_cache:
+            self.clear_cache()
         self._fake_module_functions = {}
         self._init_fake_module_functions()
 
         # Attributes set by _refresh()
-        self._modules = {}
-        self._skipped_modules = {}
-        self._fct_modules = {}
-        self._def_functions = []
-        self._open_functions = {}
         self._stubs = None
         self.fs = None
         self.fake_modules = {}
         self.unfaked_modules = {}
-        self._dyn_patcher = None
 
         # _isStale is set by tearDown(), reset by _refresh()
         self._isStale = True
+        self._dyn_patcher = None
         self._patching = False
 
+    def clear_cache(self):
+        """Clear the module cache."""
+        self.__class__.CACHED_MODULES = set()
+        self.__class__.FS_MODULES = {}
+        self.__class__.FS_FUNCTIONS = {}
+        self.__class__.FS_DEFARGS = []
+        self.__class__.SKIPPED_FS_MODULES = {}
+
     def _init_fake_module_classes(self):
         # IMPORTANT TESTING NOTE: Whenever you add a new module below, test
         # it by adding an attribute in fixtures/module_with_attributes.py
@@ -525,36 +572,33 @@ class Patcher:
 
     def _is_fs_module(self, mod, name, module_names):
         try:
-            return (inspect.ismodule(mod) and
-                    mod.__name__ in module_names
-                    or inspect.isclass(mod) and
-                    mod.__module__ in self._class_modules.get(name, []))
+            # check for __name__ first and ignore the AttributeException
+            # if it does not exist - avoids calling expansive ismodule
+            if mod.__name__ in module_names and inspect.ismodule(mod):
+                return True
         except Exception:
-            # handle cases where the module has no __name__ or __module__
-            # attribute - see #460, and any other exception triggered
-            # by inspect functions
-            return False
-
-    def _is_skipped_fs_module(self, mod, name, module_names):
+            pass
         try:
-            return (inspect.ismodule(mod) and
-                    mod.__name__ in module_names)
+            if (name in self._class_modules and
+                    mod.__module__ in self._class_modules[name]):
+                return inspect.isclass(mod)
         except Exception:
-            # handle cases where the module has no __name__ or __module__
-            # attribute - see #460, and any other exception triggered
-            # by inspect functions
+            # handle AttributeError and any other exception possibly triggered
+            # by side effects of inspect methods
             return False
 
     def _is_fs_function(self, fct):
         try:
-            return ((inspect.isfunction(fct) or
-                     inspect.isbuiltin(fct)) and
-                    fct.__name__ in self._fake_module_functions and
+            # check for __name__ first and ignore the AttributeException
+            # if it does not exist - avoids calling expansive inspect
+            # methods in most cases
+            return (fct.__name__ in self._fake_module_functions and
                     fct.__module__ in self._fake_module_functions[
-                        fct.__name__])
+                        fct.__name__] and
+                    (inspect.isfunction(fct) or inspect.isbuiltin(fct)))
         except Exception:
-            # handle cases where the function has no __name__ or __module__
-            # attribute, or any other exception in inspect functions
+            # handle AttributeError and any other exception possibly triggered
+            # by side effects of inspect methods
             return False
 
     def _def_values(self, item):
@@ -562,87 +606,86 @@ class Patcher:
         patched in top-level functions and members of top-level classes."""
         # check for module-level functions
         try:
-            if inspect.isfunction(item):
-                if item.__defaults__:
-                    for i, d in enumerate(item.__defaults__):
-                        if self._is_fs_function(d):
-                            yield item, i, d
-            elif inspect.isclass(item):
+            if item.__defaults__ and inspect.isfunction(item):
+                for i, d in enumerate(item.__defaults__):
+                    if self._is_fs_function(d):
+                        yield item, i, d
+        except Exception:
+            pass
+        try:
+            if inspect.isclass(item):
                 # check for methods in class
                 # (nested classes are ignored for now)
-                with warnings.catch_warnings():
-                    # ignore deprecation warnings, see #542
-                    warnings.filterwarnings(
-                        'ignore',
-                        category=DeprecationWarning
-                    )
-                    for m in inspect.getmembers(item,
-                                                predicate=inspect.isfunction):
-                        m = m[1]
-                        if m.__defaults__:
-                            for i, d in enumerate(m.__defaults__):
-                                if self._is_fs_function(d):
-                                    yield m, i, d
+                # inspect.getmembers is very expansive!
+                for m in inspect.getmembers(item,
+                                            predicate=inspect.isfunction):
+                    m = m[1]
+                    if m.__defaults__:
+                        for i, d in enumerate(m.__defaults__):
+                            if self._is_fs_function(d):
+                                yield m, i, d
         except Exception:
             # Ignore any exception, examples:
             # ImportError: No module named '_gdbm'
             # _DontDoThat() (see #523)
             pass
 
+    def _find_def_values(self, module_items):
+        for _, fct in module_items:
+            for f, i, d in self._def_values(fct):
+                self.__class__.FS_DEFARGS.append((f, i, d))
+
     def _find_modules(self):
         """Find and cache all modules that import file system modules.
         Later, `setUp()` will stub these with the fake file system
         modules.
         """
-
         module_names = list(self._fake_module_classes.keys()) + [PATH_MODULE]
         for name, module in list(sys.modules.items()):
             try:
-                if module in self.SKIPMODULES or not inspect.ismodule(module):
+                if (self.use_cache and module in self.CACHED_MODULES or
+                        module in self.SKIPMODULES or
+                        not inspect.ismodule(module)):
                     continue
             except Exception:
                 # workaround for some py (part of pytest) versions
                 # where py.error has no __name__ attribute
                 # see https://github.com/pytest-dev/py/issues/73
                 # and any other exception triggered by inspect.ismodule
+                if self.use_cache:
+                    self.__class__.CACHED_MODULES.add(module)
                 continue
             skipped = (any([sn.startswith(module.__name__)
                             for sn in self._skip_names]))
             module_items = module.__dict__.copy().items()
 
-            # suppress specific pytest warning - see #466
-            with warnings.catch_warnings():
-                warnings.filterwarnings(
-                    'ignore',
-                    message='The compiler package is deprecated',
-                    category=DeprecationWarning,
-                    module='py'
-                )
-                modules = {name: mod for name, mod in module_items
-                           if self._is_fs_module(mod, name, module_names)}
+            modules = {name: mod for name, mod in module_items
+                       if self._is_fs_module(mod, name, module_names)}
 
             if skipped:
                 for name, mod in modules.items():
-                    self._skipped_modules.setdefault(name, set()).add(
-                        (module, mod.__name__))
+                    self.__class__.SKIPPED_FS_MODULES.setdefault(
+                        name, set()).add((module, mod.__name__))
                 continue
 
             for name, mod in modules.items():
-                self._modules.setdefault(name, set()).add(
+                self.__class__.FS_MODULES.setdefault(name, set()).add(
                     (module, mod.__name__))
             functions = {name: fct for name, fct in
                          module_items
                          if self._is_fs_function(fct)}
 
-            # find default arguments that are file system functions
-            for _, fct in module_items:
-                for f, i, d in self._def_values(fct):
-                    self._def_functions.append((f, i, d))
-
             for name, fct in functions.items():
-                self._fct_modules.setdefault(
+                self.__class__.FS_FUNCTIONS.setdefault(
                     (name, fct.__name__, fct.__module__), set()).add(module)
 
+            # find default arguments that are file system functions
+            if self.patch_default_args:
+                self._find_def_values(module_items)
+
+            if self.use_cache:
+                self.__class__.CACHED_MODULES.add(module)
+
     def _refresh(self):
         """Renew the fake file system and set the _isStale flag to `False`."""
         if self._stubs is not None:
@@ -656,7 +699,6 @@ class Patcher:
             if hasattr(self.fake_modules[name], 'skip_names'):
                 self.fake_modules[name].skip_names = self._skip_names
         self.fake_modules[PATH_MODULE] = self.fake_modules['os'].path
-        self.fake_open = fake_filesystem.FakeFileOpen(self.fs)
         for name in self._unfaked_module_classes:
             self.unfaked_modules[name] = self._unfaked_module_classes[name]()
 
@@ -673,7 +715,14 @@ class Patcher:
             shutil._HAS_FCOPYFILE = False
 
         temp_dir = tempfile.gettempdir()
-        self._find_modules()
+        with warnings.catch_warnings():
+            # ignore deprecation warnings, see #542
+            warnings.filterwarnings(
+                'ignore',
+                category=DeprecationWarning
+            )
+            self._find_modules()
+
         self._refresh()
 
         if doctester is not None:
@@ -702,7 +751,7 @@ class Patcher:
                     reload(module)
 
     def patch_functions(self):
-        for (name, ft_name, ft_mod), modules in self._fct_modules.items():
+        for (name, ft_name, ft_mod), modules in self.FS_FUNCTIONS.items():
             method, mod_name = self._fake_module_functions[ft_name][ft_mod]
             fake_module = self.fake_modules[mod_name]
             attr = method.__get__(fake_module, fake_module.__class__)
@@ -710,18 +759,18 @@ class Patcher:
                 self._stubs.smart_set(module, name, attr)
 
     def patch_modules(self):
-        for name, modules in self._modules.items():
+        for name, modules in self.FS_MODULES.items():
             for module, attr in modules:
                 self._stubs.smart_set(
                     module, name, self.fake_modules[attr])
-        for name, modules in self._skipped_modules.items():
+        for name, modules in self.SKIPPED_FS_MODULES.items():
             for module, attr in modules:
                 if attr in self.unfaked_modules:
                     self._stubs.smart_set(
                         module, name, self.unfaked_modules[attr])
 
     def patch_defaults(self):
-        for (fct, idx, ft) in self._def_functions:
+        for (fct, idx, ft) in self.FS_DEFARGS:
             method, mod_name = self._fake_module_functions[
                 ft.__name__][ft.__module__]
             fake_module = self.fake_modules[mod_name]
@@ -761,7 +810,7 @@ class Patcher:
             sys.meta_path.pop(0)
 
     def unset_defaults(self):
-        for (fct, idx, ft) in self._def_functions:
+        for (fct, idx, ft) in self.FS_DEFARGS:
             new_defaults = []
             for i, d in enumerate(fct.__defaults__):
                 if i == idx:
@@ -769,7 +818,7 @@ class Patcher:
                 else:
                     new_defaults.append(d)
             fct.__defaults__ = tuple(new_defaults)
-        self._def_functions = []
+        # self._def_functions = []
 
     def pause(self):
         """Pause the patching of the file system modules until `resume` is
