@@ -61,17 +61,10 @@ class TestCase(unittest.TestCase):
             expression(*args, **kwargs)
             self.fail('No exception was raised, OSError expected')
         except OSError as exc:
-            self.assertEqual(subtype, exc.errno)
-
-    def assert_equal_paths(self, actual, expected):
-        if self.is_windows:
-            self.assertEqual(actual.replace('\\\\?\\', ''),
-                             expected.replace('\\\\?\\', ''))
-        elif self.is_macos:
-            self.assertEqual(actual.replace('/private/var/', '/var/'),
-                             expected.replace('/private/var/', '/var/'))
-        else:
-            self.assertEqual(actual, expected)
+            if isinstance(subtype, list):
+                self.assertIn(exc.errno, subtype)
+            else:
+                self.assertEqual(subtype, exc.errno)
 
 
 class RealFsTestMixin:
@@ -243,15 +236,18 @@ class RealFsTestMixin:
                 raise unittest.SkipTest(
                     'Skipping because FakeFS does not match real FS')
 
-    def symlink_can_be_tested(self):
+    def symlink_can_be_tested(self, force_real_fs=False):
         """Used to check if symlinks and hard links can be tested under
         Windows. All tests are skipped under Windows for Python versions
         not supporting links, and real tests are skipped if running without
         administrator rights.
         """
-        if not TestCase.is_windows or not self.use_real_fs():
+        if (not TestCase.is_windows or
+                (not force_real_fs and not self.use_real_fs())):
             return True
         if TestCase.symlinks_can_be_tested is None:
+            if force_real_fs:
+                self.base_path = tempfile.mkdtemp()
             link_path = self.make_path('link')
             try:
                 self.os.symlink(self.base_path, link_path)
@@ -259,12 +255,14 @@ class RealFsTestMixin:
                 self.os.remove(link_path)
             except (OSError, NotImplementedError):
                 TestCase.symlinks_can_be_tested = False
+            if force_real_fs:
+                self.base_path = None
         return TestCase.symlinks_can_be_tested
 
-    def skip_if_symlink_not_supported(self):
+    def skip_if_symlink_not_supported(self, force_real_fs=False):
         """If called at test start, tests are skipped if symlinks are not
         supported."""
-        if not self.symlink_can_be_tested():
+        if not self.symlink_can_be_tested(force_real_fs):
             raise unittest.SkipTest(
                 'Symlinks under Windows need admin privileges')
 
@@ -347,6 +345,29 @@ class RealFsTestMixin:
                 if old_base_path is not None:
                     self.setUpFileSystem()
 
+    def assert_equal_paths(self, actual, expected):
+        if self.is_windows:
+            actual = str(actual).replace('\\\\?\\', '')
+            expected = str(expected).replace('\\\\?\\', '')
+            if os.name == 'nt' and self.use_real_fs():
+                # work around a problem that the user name, but not the full
+                # path is shown as the short name
+                self.assertEqual(self.path_with_short_username(actual),
+                                 self.path_with_short_username(expected))
+            else:
+                self.assertEqual(actual, expected)
+        elif self.is_macos:
+            self.assertEqual(str(actual).replace('/private/var/', '/var/'),
+                             str(expected).replace('/private/var/', '/var/'))
+        else:
+            self.assertEqual(actual, expected)
+
+    def path_with_short_username(self, path):
+        components = path.split(os.sep)
+        if len(components) >= 3:
+            components[2] = components[2][:6].upper() + '~1'
+        return os.sep.join(components)
+
 
 class RealFsTestCase(TestCase, RealFsTestMixin):
     """Can be used as base class for tests also running in the real
